]> Dogcows Code - chaz/tar/blobdiff - src/rmt.c
(dump_file): Do not diagnose a file that the parent directory
[chaz/tar] / src / rmt.c
index 30d665f8746f17581c99423a9ebc2339b97393ca..0215bffb8a577855380ecb7068866bd9eaf80780 100644 (file)
--- a/src/rmt.c
+++ b/src/rmt.c
    derived from this software without specific prior written permission.
    THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED
    WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
-   MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.  */
+   MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  */
 
 #include "system.h"
 #include "safe-read.h"
 
+#include <getopt.h>
 #include <sys/socket.h>
 
 #ifndef EXIT_FAILURE
@@ -51,15 +52,15 @@ const char *program_name;
 static int tape = -1;
 
 /* Buffer containing transferred data, and its allocated size.  */
-static char *record_buffer = NULL;
-static size_t allocated_size = 0;
+static char *record_buffer;
+static size_t allocated_size;
 
 /* Buffer for constructing the reply.  */
 static char reply_buffer[BUFSIZ];
 
 /* Debugging tools.  */
 
-static FILE *debug_file = NULL;
+static FILE *debug_file;
 
 #define        DEBUG(File) \
   if (debug_file) fprintf(debug_file, File)
@@ -153,7 +154,7 @@ prepare_record_buffer (size_t size)
 
   record_buffer = malloc (size);
 
-  if (record_buffer == NULL)
+  if (! record_buffer)
     {
       DEBUG (_("rmtd: Cannot allocate buffer space\n"));
 
@@ -252,6 +253,34 @@ decode_oflag (char const *oflag_string)
   return symbolic_oflag;
 }
 
+static struct option const long_opts[] =
+{
+  {"help", no_argument, 0, 'h'},
+  {"version", no_argument, 0, 'v'},
+  {0, 0, 0, 0}
+};
+
+static void
+usage (int status)
+{
+  if (status != EXIT_SUCCESS)
+    fprintf (stderr, _("Try `%s --help' for more information.\n"),
+            program_name);
+  else
+    {
+      printf (_("\
+Usage: %s [OPTION]\n\
+Manipulate a tape drive, accepting commands from a remote process.\n\
+\n\
+  --version  Output version info.\n\
+  --help  Output this help.\n"),
+             program_name);
+      fputs (_("\nReport bugs to <bug-tar@gnu.org>.\n"), stdout);
+    }
+
+  exit (status);
+}
+
 /*---.
 | ?  |
 `---*/
@@ -260,10 +289,10 @@ int
 main (int argc, char *const *argv)
 {
   char command;
-  long status;
+  ssize_t status;
 
-  /* FIXME: Localisation is meaningless, unless --help and --version are
-     locally used.  Localisation would be best accomplished by the calling
+  /* FIXME: Localization is meaningless, unless --help and --version are
+     locally used.  Localization would be best accomplished by the calling
      tar, on messages found within error packets.  */
 
   program_name = argv[0];
@@ -271,25 +300,45 @@ main (int argc, char *const *argv)
   bindtextdomain (PACKAGE, LOCALEDIR);
   textdomain (PACKAGE);
 
-  /* FIXME: Implement --help and --version as for any other GNU program.  */
+  switch (getopt_long (argc, argv, "", long_opts, NULL))
+    {
+    default:
+      usage (EXIT_FAILURE);
+      
+    case 'h':
+      usage (EXIT_SUCCESS);
+      
+    case 'v':
+      printf ("rmt (GNU %s) %s\n%s\n%s\n", PACKAGE, VERSION,
+             "Copyright 1999 Free Software Foundation, Inc.",
+             _("\
+This program comes with NO WARRANTY, to the extent permitted by law.\n\
+You may redistribute it under the terms of the GNU General Public License;\n\
+see the file named COPYING for details."));
+      return EXIT_SUCCESS;
+
+    case -1:
+      break;
+    }
 
-  argc--, argv++;
-  if (argc > 0)
+  if (optind < argc)
     {
-      debug_file = fopen (*argv, "w");
+      if (optind != argc - 1)
+       usage (EXIT_FAILURE);
+      debug_file = fopen (argv[optind], "w");
       if (debug_file == 0)
        {
          report_numbered_error (errno);
          exit (EXIT_FAILURE);
        }
-      setbuf (debug_file, NULL);
+      setbuf (debug_file, 0);
     }
 
 top:
   errno = 0;                   /* FIXME: errno should be read-only */
   status = 0;
   if (safe_read (STDIN_FILENO, &command, 1) != 1)
-    exit (EXIT_SUCCESS);
+    return EXIT_SUCCESS;
 
   switch (command)
     {
@@ -437,9 +486,9 @@ top:
        status = safe_read (tape, record_buffer, size);
        if (status < 0)
          goto ioerror;
-       sprintf (reply_buffer, "A%ld\n", status);
+       sprintf (reply_buffer, "A%ld\n", (long) status);
        full_write (STDOUT_FILENO, reply_buffer, strlen (reply_buffer));
-       full_write (STDOUT_FILENO, record_buffer, (size_t) status);
+       full_write (STDOUT_FILENO, record_buffer, status);
        goto top;
       }
 
@@ -512,10 +561,10 @@ top:
 
          if (ioctl (tape, MTIOCGET, (char *) &operation) < 0)
            goto ioerror;
-         status = sizeof (operation);
-         sprintf (reply_buffer, "A%ld\n", status);
+         status = sizeof operation;
+         sprintf (reply_buffer, "A%ld\n", (long) status);
          full_write (STDOUT_FILENO, reply_buffer, strlen (reply_buffer));
-         full_write (STDOUT_FILENO, (char *) &operation, sizeof (operation));
+         full_write (STDOUT_FILENO, (char *) &operation, sizeof operation);
        }
 #endif
        goto top;
@@ -529,9 +578,9 @@ top:
     }
 
 respond:
-  DEBUG1 ("rmtd: A %ld\n", status);
+  DEBUG1 ("rmtd: A %ld\n", (long) status);
 
-  sprintf (reply_buffer, "A%ld\n", status);
+  sprintf (reply_buffer, "A%ld\n", (long) status);
   full_write (STDOUT_FILENO, reply_buffer, strlen (reply_buffer));
   goto top;
 
This page took 0.024468 seconds and 4 git commands to generate.